		/* ado-file to make prediction based on the model  
    c(i,t) = popu(i,t)*exp( alpha(i) + beta*t )
    Hakulinen T, Dyba T. Precision of incidence predictions based on Poisson distributed observations. Statistics in Medicine 13: 1513--23, 1994
    
    Example of using it predmaap case_vari person-year_vari , age(age_vari) peri(peri_vari) ppn(prognostic-population_column-matrix) pps(prognostic-standard_column-matrix) tp(time-of-prediction_number) 
    you can also add optionally graph option and title(Your text for graph) if you want to get the graphs of estimated predictions, the graphs will be saved in the files named: predmaap_case.gph and predmaap_adju.gph, then the command will look like:
    predmaap case_vari person-year_vari , age(age_vari) peri(peri_vari) ppn(prognostic-population_column-matrix) pps(prognostic-standard_column-matrix) tp(time-of-prediction_number) graph title(Your text for graph)
    you can also use if and in statement for the command (predmaap case_vari person-year_vari if... in..., ...) but if you do so don't forget to adjust matrices prognostic-population_column-matrix and prognostic-standard_column-matrix for the new number of age groups
    */

    program define predmaap /**/
    version 8.0
    
    syntax varlist(min=2 max=2) [if] [in] , age(string) peri(string) [cons(numlist) ppn(string) pps(string) tp(real 1) graph title(string) agegr(string) file(string)]
    tokenize "`varlist'"
    args case popu
		tempname h1 s1 s2 ppn_ pps_
		tempvar tperi ycase predcase obseadju predadju pil piu obseinci predinci

    preserve
    
    display     " "
    capture keep  `if' `in'
    //Changing age code from whatever it is to 1,2,3,4,..
    qui inspect `age'
    local nage=r(N_unique) // how many age-groups
    qui tabu `age', gen(_nage)
    local i = 1
    while `i' <= `nage' {
    qui replace `age' = `i' if _nage`i'==1
     local i = `i'+1
    } 

		// for age groups with constant incidence
		if "`cons'" != "" {
		g `tperi' = `peri'
		local cl = wordcount("`cons'") // number of age groups with constant incidence
			forvalues i = 1/`cl'  {	// trick with peri==0 for each age group
				local ca = word("`cons'",`i')	// index of the age group
				replace `tperi' = 0 if `age'==`ca'
			}
		}
    
    // To define own parametrization
    local i = 1
    while `i' <= `nage' {
    g _age_`i'        = ( `age' == `i' )
	  if "`cons'" != "" g _age`i'Xperi  = ( `age' == `i' )*`tperi'
			else						g _age`i'Xperi  = ( `age' == `i' )*`peri'
    local i = `i' + 1
    } 
    
    glm `case' _age_* _age*X* ,fami(pois)  link(log) lnoff(`popu') nocon iterate(1000)    /*fitting */
    display     " "
    di " p-value for Pearsons' chi2= " chi2tail(e(df),e(deviance_p))
    
    //PREDICTION
    if "`ppn'" != "" {
			matrix `ppn_' = `ppn'
			matrix `pps_' = `pps'

		global over_rati = 0
		// possible overdispersion
    if chi2tail(e(df),e(deviance_p)) < 0.05   {
			di " Results adjusted for overdispersion, dispersion factor=" e(deviance_p)/e(df)
			global over_rati = e(deviance_p)/e(df)
		}
    matr `h1' = `pps_'' * J( rowsof(`pps_'), 1, 1)  /*norming standard population*/
    scala `h1'=`h1'[1,1]                          /*norming standard population*/
    matr `pps_' = `pps_'/`h1'                       /*norming standard population*/
    
    //ouput after fitting
    matr   V  = get( VCE )      //co-variance matrix
		global over_rati = 0 // to save overdispersion rate if overdispersion
		if chi2tail(e(df),e(deviance_p)) < 0.05 {
			matr V=V*(e(deviance_p)/e(df)) /*correction for possible overdispersion*/
			global over_rati = (e(deviance_p)/e(df)) // to save overdispersion rate if overdispersion
		}		
		matr   PE = get( _b )       // parameter vector 
    matr   PE = PE'

		// Part to include predictions for age groups with constant incidence for PE
		if "`cons'" != "" {
			local cl = wordcount("`cons'") // cl<-number of age groups with constant incidence
				forvalues age2 = 1/`cl'  {	// adjusting PE and V <-- dropping parameters by SA
					local ca = word("`cons'",`age2')	// ca<-current age, index of the age group
						if PE[(`nage'+`ca'),1]==. {
							mat PE = PE\0
							continue
						}
					matr PE = PE[1..(`nage'+`ca'-1),1]\0\PE[(`nage'+`ca')...,1]
				}
			local cl = wordcount("`cons'") // cl<-number of age groups with constant incidence
			forvalues age2 = 1/`cl'  {	// adjusting PE and V <-- dropping parameters by SA
				local ca = word("`cons'",`age2')	// ca<-current age, index of the age group
				if V[1, (`nage'+`ca')]==. {
					mat V = V, J(rowsof(V),1,0) // dodaje kolumne zer gdy na samym koncu
					mat V = V\ J(1,colsof(V),0) // dodaje wiersz zer gdy na samym koncu
					continue
				}
				matr V = V[1..., 1..(`nage'+`ca'-1)], J(rowsof(V), 1, 0), V[1..., (`nage'+`ca')...] // dodaje kolumne zer w srodku
				matr V = V[1..(`nage'+`ca'-1), 1...]\ J(1, colsof(V), 0)\ V[(`nage'+`ca')..., 1...] // dodaje wiersz zer w srodku
			}
		}
		local   na  = rowsof(PE)/2  // number of age groups, MUSI BYC TUTAJ
    
    // Creating matrix of results with names of rows and columns
    mat OUTMAAP=J( `na' + 2, 6, .)
    local outname "age1"
    local i  2
    while `i' <=  `na' {
    local outname `outname'  age`i'
    local  i  =  `i' + 1
    }
    local outname   `outname' ages-combined  adj-rate/10^5
    matrix rownames  OUTMAAP  = `outname'  
    matrix colnames   OUTMAAP   =  inci/10^5 no-cases  se(expe-value)  se(pred-value)  Low95%PredInte   Up
    
    //  Prediction of age-specific results
    local i=1
    while `i' <=  `na' {
    mat    OUTMAAP[`i', 1] = exp(PE[`i',1] + PE[`na' + `i',1] * `tp')	/*incidence */
    mat    OUTMAAP[`i', 2] = OUTMAAP[`i', 1] * `ppn_'[`i',1]	        	/*cases    */
    scal `s1' = OUTMAAP[`i',2]						/*to calculate se of age-specific pred */
    scal `s2' = OUTMAAP[`i',2]*`tp'						/*to calculate se of age-specific pred */
    mat   OUTMAAP[`i' , 3] =  sqrt(  V[ `i' , `i' ]*(`s1'^2) + V[`i', `na' + `i']*`s1'*`s2'*2 + V[`na' + `i', `na' + `i'] * (`s2'^2)  )  /*se of expe age-specific pred*/
    mat   OUTMAAP[`i' , 4] =  sqrt(  OUTMAAP[`i' , 3]^2  +  OUTMAAP[`i', 2]    ) /*se of pred age-specific pred*/
    mat   OUTMAAP[`i' , 5] =  OUTMAAP[`i' , 2] - 1.96*OUTMAAP[`i' , 4]  /*95% PredInter of age-specific pred*/
    mat   OUTMAAP[`i' , 6] =  OUTMAAP[`i' , 2] + 1.96*OUTMAAP[`i' , 4]  /*95% PredInter of age-specific pred*/
    local  i  =  `i' + 1
    }
    // OVERALL RESULTS
    mat   OUTMAAP[`na'+1 , 2]  =  OUTMAAP[1..`na', 2]' * J(`na',1,1) /* exp for cases overall  */
    matrix  `h1' = `ppn_'' * J(`na',1,1)				 /* inci          overall  */
    scalar `h1' = `h1'[1,1]                                          /* inci           overall */
    mat   OUTMAAP[`na'+1,  1]  =  OUTMAAP[`na'+1 , 2] /`h1'          /* inci           overall */
    mat   OUTMAAP[`na'+2,  1]  =  OUTMAAP[1..`na', 1]' * `pps_'       /* exp for adj-rate       */
    // stanard errors by delta method for cases
    mat  `h1' = OUTMAAP[1..`na',2] \ (OUTMAAP[1..`na',2]*`tp')				 	/*partial derivatives for cases */
		// part-deriv for cons cases 
		if "`cons'" != "" {
		local cl = wordcount("`cons'") 
			forvalues i = 1/`cl'  {	
				local ca = word("`cons'",`i')	// index of the age group in cons
				mat `h1'[`na'+`ca',1] = 0
			}
		}
		mat  OUTMAAP[`na'+1, 3] = sqrt(trace(`h1'*(`h1'')*V))           	/* se of exp for cases         */
    mat  OUTMAAP[`na'+1, 4] = sqrt(OUTMAAP[`na'+1, 3]^2+OUTMAAP[`na'+1,2])  /* se of prediction for cases  */
    **for adj-rate
    mat  `h1' = vecdiag(`pps_'*OUTMAAP[1..`na',1]') 	/*partial derivatives for adj-rate*/
    mat `h1'= `h1''\((`h1'*`tp')')	/*partial derivatives for adj-rate*/
		// part-deriv for cons adj
		if "`cons'" != "" {
		local cl = wordcount("`cons'")
			forvalues i = 1/`cl'  {	
				local ca = word("`cons'",`i')	// index of the age group
				mat `h1'[`na'+`ca',1] = 0
			}
		}
    mat  OUTMAAP[`na'+2, 3] = sqrt(trace(`h1'*(`h1'')*V))  			/* se of exp for adj-rate*/
    //loop necessary to calculate se of predi for adjrate <-- STATA cannot divide matrices routinely
    local i=1
    local h1=0
    while `i' <=  `na' {
    local h1=`h1'+ `pps_'[`i',1]^2/`ppn_'[`i',1]*OUTMAAP[`i',1]
    local  i  =  `i' + 1
    }
    mat OUTMAAP[`na'+2, 4] = sqrt(OUTMAAP[`na'+2, 3]^2+`h1')  	/* se of prediction for adj-rate*/

		//95% Prediction Intervals
    mat  OUTMAAP[`na'+1,5]=OUTMAAP[`na'+1,2]-1.96*OUTMAAP[`na'+1,4] /*95%PI for cases   */
    mat  OUTMAAP[`na'+1,6]=OUTMAAP[`na'+1,2]+1.96*OUTMAAP[`na'+1,4] /*95%PI for cases   */
    mat  OUTMAAP[`na'+2,5]=OUTMAAP[`na'+2,1]-1.96*OUTMAAP[`na'+2,4] /*95%PI for adj-rate*/
    mat  OUTMAAP[`na'+2,6]=OUTMAAP[`na'+2,1]+1.96*OUTMAAP[`na'+2,4] /*95%PI for adj-rate*/
    
    //All results for age-specific incidences and adj-rates per 10^5 person-years
    local i=1
    while `i' <=  (`na'+1) {
    mat OUTMAAP[`i',1]=OUTMAAP[`i',1]*10^5
		local  i  =  `i' + 1
    }
    local i=1
    while `i' <=  6 {
    mat OUTMAAP[`na'+2,`i']=OUTMAAP[`na'+2,`i']*10^5
    local  i  =  `i' + 1
    }
    
    //Correction for minus pi
    local i=1
    while `i' <=  (`na'+2) {
    if OUTMAAP[`i',5] <0 mat OUTMAAP[`i',5]=0
    if OUTMAAP[`i',6] <0 mat OUTMAAP[`i',6]=0
    local  i  =  `i' + 1
    }
    
    display     " "
    display     " Prediction based on the model case(i,t) = popu(i,t)*exp(alpha(i) + beta(i)*t)"
    display     " Prediction for the period=" `tp' ", results below saved in the matrix OUTMAAP"
    mat list OUTMAAP, f(%6.2f)
    }  // the end of if "`ppn_'" != "" {
	
	//// GRAPHS
	if "`graph'" != "" {
	di " "
	quietly {
	// Age-specific predictions
	g `obseinci' = `case'/`popu'*10^5
	predict `predinci'
	replace `predinci'=`predinci'/`popu'*10^5
	g `pil'=.
	g `piu'=.
	local iage = 1
	while `iage' <= `nage' {
		local nobs = _N+1 // adding tp for a given age-group
		set obs `nobs'
		replace `peri' = `tp'   in l
		replace `predinci' = OUTMAAP[`iage',1]   in l
		replace `age'  = `iage' in l
		replace `pil'  = OUTMAAP[`iage',5]/`ppn_'[`iage',1]*10^5 in l // prediction interval down all
		replace `piu'  = OUTMAAP[`iage',6]/`ppn_'[`iage',1]*10^5 in l // prediction interval up all
		local iage = `iage' + 1
	}
	sort `age' `peri'
	twoway (line `obseinci' `peri' if `peri'!=`tp') ///
         (line `predinci' `peri' if `peri'!=`tp', clpattern(dash)) ///
			   (scatter `predinci' `peri' if `peri'==`tp', msymbol(+)) ///
			   (rcap `pil' `piu' `peri' if `peri'==`tp'), ///
			  by(`age', title(`title')) xtitle(Period) ytitle(Inci/10^5) saving(predmaap_age,replace) ///
				legend( label(1 "observed") label(2 "fitted") order(1 2)) ylabel( ,angle(horizontal))
		if "`agegr'" != "" {
		replace `predinci' = OUTMAAP[`agegr',1]   in l
		replace `pil'  = OUTMAAP[`agegr',5]/`ppn_'[`agegr',1]*10^5 in l // prediction interval down all
		replace `piu'  = OUTMAAP[`agegr',6]/`ppn_'[`agegr',1]*10^5 in l // prediction interval up all
		twoway (line `obseinci' `peri' if `peri'!=`tp' & `age'==`agegr') ///
         (line `predinci' `peri' if `peri'!=`tp' & `age'==`agegr', clpattern(dash)) ///
			   (scatter `predinci' `peri' if `peri'==`tp' & `age'==`agegr', msymbol(+)) ///
			   (rcap `pil' `piu' `peri' if `peri'==`tp' & `age'==`agegr'), ///
					title("`title'  age=`agegr'") xtitle(Period) ytitle(Inci/10^5) ysc(r(0)) saving(predmaap_age`agegr',replace) ///
					legend( label(1 "observed") label(2 "fitted") order(1 2)) ylabel( ,angle(horizontal))
		}
		drop if `peri' == `tp'

	// Cases
	replace   `predinci' = `predinci'/10^5
	g   `predcase' = `predinci'*`popu' // predicted cases, musze podzielic na 10^5
	g   `obseadju'   = `pps_'[`age',1]*`case'/`popu'*(10^5) // observed adj-rate
	g   `predadju' = `predinci'*`pps_'[`age',1]*(10^5)       // to calculate pred-hist age-adj
	collapse (sum) `case' `predcase' `predadju' `obseadju', by(`peri')
	local nobs = _N+1 // adding tp
	set obs `nobs'
	replace `predcase' = OUTMAAP[`na'+1 , 2] in `nobs'
	replace      `peri' =`tp'  in `nobs' //
	g `pil' = OUTMAAP[`na'+1,5] // prediction interval down
	g `piu' = OUTMAAP[`na'+1,6] // prediction interval up
	twoway (line `case' `peri' if `peri'!=`tp') (line `predcase' `peri' if `peri'!=`tp', clpattern(dash)) (rcap `pil' `piu' `peri' if `peri'==`tp') ///
	(scatter `predcase' `peri' if `peri'==`tp', msymbol(+)), ysc(r(0)) ylabel( ,angle(horizontal)) ///
	legend( label(1 "observed") label(2 "fitted") ) ytitle(Number of cases) xtitle(Period) saving(predmaap_case,replace) /*ylabel(0(`s2')`s1')*/ ///
	title(`title')  legend( label(1 "observed") label(2 "fitted") order(1 2)) // graph for number of cases
	// Adj-rate
	replace   `predadju' = OUTMAAP[`nage'+2,  1] in `nobs' // predicted value for adj-rate
	replace      `peri' =`tp'  in `nobs'
	replace `pil' = OUTMAAP[`na'+2,5] // prediction interval down
	replace `piu' = OUTMAAP[`na'+2,6] // prediction interval up
	local s1=round(OUTMAAP[`na'+2,6]*1.3, 1) // numbers gora the graph
	local s2=round(`s1'/4)                // numbers gor the graph
	twoway (line `obseadju'  `peri' if `peri'!=`tp')  (line `predadju' `peri' if `peri'!=`tp', clpattern(dash)) ///
	(rcap `pil' `piu' `peri' if `peri'==`tp') (scatter `predadju' `peri' if `peri'==`tp', msymbol(+)), ///
	ytitle(Adjusted-Rate/10^5) xtitle(Period) ylabel( ,angle(horizontal)) saving(predmaap_adju,replace) ///
	ysc(r(0)) title(`title') legend( label(1 "observed") label(2 "fitted") order(1 2)) 
	if "`agegr'" != "" noi di "Graphs saved in the files: predmaap_case.gph predmaap_adju.gph predmaap_age.gph predmaap_age`agegr'.gph"
		else noi di "Graphs saved in the files: predmaap_case.gph predmaap_adju.gph predmaap_age.gph.gph"
	}
	} // graphs

	if "`file'" != "" {
		drop _all
		mat coln OUTMAAP = inci10_5 cases se_expe se_pred pred95_l pred95_u
		qui svmat OUTMAAP, n(col)
		qui g str10 what = ""
		local i = 1
		while `i' <= `nage' {
			qui replace what = "age`i'" in `i'
			local i = `i'+1
	}
	loc  i = _N-1
	qui replace what = "all_ages" in `i'
	loc  i = `i'+1
	qui replace what = "adj10_5" in `i'
	order what
	di "Results saved in the file `file'.dta"
	qui save `file', replace
		}
    restore
	end
