  /* ado-file to make prediction based on the model  
  c(i,t) = popu(i,t)*exp( alpha(i) + beta*t ) 
  Hakulinen T, Dyba T. Precision of incidence predictions based on Poisson distributed observations. Statistics in Medicine 13: 1513--23, 1994
  Example of using it predmap case_vari person-year_vari , age(age_vari) peri(peri_vari) ppn(prognostic-population_column-matrix) pps(prognostic-standard_column-matrix) tp(time-of-prediction_number) 
  you can also add optionally graph option and title(Your text for graph) if you want to get the graphs of estimated predictions, the graphs will be saved in the files named: predmap_case.gph and predmap_adju.gph, then the command will look like:
  predmap case_vari person-year_vari , age(age_vari) peri(peri_vari) ppn(prognostic-population_column-matrix) pps(prognostic-standard_column-matrix) tp(time-of-prediction_number) graph title(Your text for graph)
  you can also use if and in statement for the command (predmap case_vari person-year_vari if... in..., ...) but if you do so don't forget to adjust matrices prognostic-population_column-matrix and prognostic-standard_column-matrix for the new number of age groups */
  
  program define predmap /**/
  version 8.0
  
  syntax varlist(min=2 max=2) [if] [in] , age(string) peri(string) [cons(numlist) ppn(string) pps(string) tp(real 1) graph title(string) agegr(string) file(string)]
  tokenize "`varlist'"
  args case popu
	tempname h1 s1 s2 tperi predcase obseadju predadju pil piu obseinci predinci ppn_ pps_
  preserve
  
  display     " "
  capture keep  `if' `in'
  //Changing age code from whatever it is to 1,2,3,4,..
  qui inspect `age'
  local nage=r(N_unique) // how many age-groups
  qui tabu `age', gen(_nage)
  local i = 1
  while `i' <= `nage' {
  qui replace `age' = `i' if _nage`i'==1
  local i = `i'+1
  } 

	g `tperi' = `peri'  // tperi for estimation, peri for figures
  	// The part to include predictions for age groups with constant incidence
  	if "`cons'" != "" {
  		local cl = wordcount("`cons'") // number of age groups with constant incidence
  		forvalues i = 1/`cl'  {	// trick with peri==0 for each age group
  				local ca = word("`cons'",`i')	// index of the age group
  				replace `peri' = 0 if `age'==`ca'
  		}
  	}

  //To define own parametrization
  local i = 1
  while `i' <= `nage' {
  g _age_`i'        = ( `age' == `i' )
  local i = `i' + 1
  } 
  glm `case' _age_*  `peri'  ,fami(pois) link(log) lnoff(`popu') nocon iterate(1000)    /*fitting */
  display     " "
  di " p-value for Pearsons' chi2= " chi2tail(e(df),e(deviance_p))

  //PREDICTION
  if "`ppn'" != "" {
			matrix `ppn_' = `ppn'
			matrix `pps_' = `pps'
  //possible overdispersion
  if chi2tail(e(df),e(deviance_p)) < 0.05 display " Results adjusted for overdispersion, dispersion factor=" e(deviance_p)/e(df)
  matr `h1' = `pps_'' * J( rowsof(`pps_'), 1, 1)  /*norming standard population*/
  scala `h1'=`h1'[1,1]                          /*norming standard population*/
  matr `pps_' = `pps_'/`h1'                       /*norming standard population*/
  //ouput after fitting*/
  matr   V  = get( VCE )      /* co-variance matrix */
  if chi2tail(e(df),e(deviance_p)) < 0.05 matr V=V*(e(deviance_p)/e(df)) /*correction for possible overdispersion*/
  matr   PE = get( _b )       /* parameter vector   */
  matr   PE = PE'
  local   na  = rowsof(PE)-1  /* number of age groups */
  
  //Creating matrix of results with names of rows and columns
  mat OUTMAP=J( `na' + 2, 6, .)
  local outname "age1"
  local i  2
  while `i' <=  `na' {
  local outname `outname'  age`i'
  local  i  =  `i' + 1
  }
  local outname   `outname' ages-combined  adj-rate/10^5
  matrix rownames  OUTMAP  = `outname'  
  matrix colnames   OUTMAP   =  inci/10^5 no-cases  se(expe-value)  se(pred-value)  Low95%PredInte   Up
  
  //Calculating prediction
	//age-specific results
  local i= 1
	while `i' <=  `na' { //*poczatek wyrzuca beta*t dla constant grup wiekowych
	local mn = 1 // *mnoznik 0/1   
 	if "`cons'" != "" { // *by tp nie znieksztalcilo wyniku dla stalych grup wiekowych
	local cl = wordcount("`cons'") // *number of age groups with constant incidence
  	forvalues j = 1/`cl'  {	// *trick with peri==0 for each age group
  		local ca = word("`cons'",`j')	// *index of the age group
		if `i' == `ca' local mn = 0 // gdy stala grupa to mnoznik jest 0
  	}
  }
	mat    OUTMAP[`i', 1] = exp(PE[`i',1] + PE[`na' + 1,1] * `tp' *`mn')	/*incidence */
  mat    OUTMAP[`i', 2] = OUTMAP[`i', 1] * `ppn_'[`i',1]	        	/*cases    */
  scal `s1' = OUTMAP[`i',2]							/*to calculate se of age-specific pred */
  scal `s2' =  OUTMAP[`i',2]*`tp'						/*to calculate se of age-specific pred */
  mat   OUTMAP[`i' , 3] =  sqrt(  V[ `i' , `i' ]*(`s1'^2) + V[`i', `na' + 1]*`s1'*`s2'*2 + V[`na' + 1, `na' + 1] * (`s2'^2)  )  /*se of expe age-specific pred*/
  mat   OUTMAP[`i' , 4] =  sqrt(  OUTMAP[`i' , 3]^2  +  OUTMAP[`i', 2]    ) /*se of pred age-specific pred*/
  mat   OUTMAP[`i' , 5] =  OUTMAP[`i' , 2] - 1.96*OUTMAP[`i' , 4]  		  /*95% PredInter of age-specific pred*/
  mat   OUTMAP[`i' , 6] =  OUTMAP[`i' , 2] + 1.96*OUTMAP[`i' , 4]  		  /*95% PredInter of age-specific pred*/
	local  i  =  `i' + 1
  }
  
  //OVERALL RESULTS
  mat   OUTMAP[`na'+1 , 2]  =  OUTMAP[1..`na', 2]' * J(`na',1,1)/* exp for cases overall  */
  matrix  `h1' = `ppn_'' * J(`na',1,1)				 		/* inci   overall  */
  scalar `h1' = `h1'[1,1]                                        /* inci           overall */
  mat   OUTMAP[`na'+1,  1]  =  OUTMAP[`na'+1 , 2] /`h1'          /* inci           overall */
  mat   OUTMAP[`na'+2,  1]  =  OUTMAP[1..`na' , 1]' * `pps_'      /* exp for adj-rate       */
  //stanard errors by delta method for cases
  mat  `h1' = OUTMAP[1..`na',2] \ (OUTMAP[`na'+1,2]*`tp')		 	/*partial derivatives for cases */
  mat  OUTMAP[`na'+1, 3] = sqrt(trace(`h1'*(`h1'')*V))           	/* se of exp for cases         */
  mat  OUTMAP[`na'+1, 4] = sqrt(OUTMAP[`na'+1, 3]^2+OUTMAP[`na'+1,2])  /* se of prediction for cases  */
  // for adj-rate
  mat  `h1' = vecdiag(`pps_'*OUTMAP[1..`na',1]') 	/*partial derivatives for adj-rate*/
  mat `h1'= `h1''\(OUTMAP[`na'+2,1]*`tp')		/*partial derivatives for adj-rate*/
  mat  OUTMAP[`na'+2, 3] = sqrt(trace(`h1'*(`h1'')*V))  			/* se of exp for adj-rate*/
  // loop necessary to calculate se of predi for adjrate <-- STATA cannot divide matrices routinely
  local i=1
  local h1=0
  while `i' <=  `na' {
  local h1=`h1'+ `pps_'[`i',1]^2/`ppn_'[`i',1]*OUTMAP[`i',1]
  local  i  =  `i' + 1
  }
  mat OUTMAP[`na'+2, 4] = sqrt(OUTMAP[`na'+2, 3]^2+`h1')  	/* se of prediction for adj-rate*/
  // 95% Prediction Intervals
  mat  OUTMAP[`na'+1,5]=OUTMAP[`na'+1,2]-1.96*OUTMAP[`na'+1,4] /*95%PI for cases   */
  mat  OUTMAP[`na'+1,6]=OUTMAP[`na'+1,2]+1.96*OUTMAP[`na'+1,4] /*95%PI for cases   */
  mat  OUTMAP[`na'+2,5]=OUTMAP[`na'+2,1]-1.96*OUTMAP[`na'+2,4] /*95%PI for adj-rate*/
  mat  OUTMAP[`na'+2,6]=OUTMAP[`na'+2,1]+1.96*OUTMAP[`na'+2,4] /*95%PI for adj-rate*/
  //Results for age-specific incidences and adj-rates per 10^5 person-years
  local i=1
  while `i' <=  (`na'+1) {
  mat OUTMAP[`i',1]=OUTMAP[`i',1]*10^5
  local  i  =  `i' + 1
  }
  local i=1
  while `i' <=  6 {
  mat OUTMAP[`na'+2,`i']=OUTMAP[`na'+2,`i']*10^5
  local  i  =  `i' + 1
  }
  //Correction for minus pi
  local i=1
  while `i' <=  (`na'+2) {
  if OUTMAP[`i',5] <0 mat OUTMAP[`i',5]=0
  if OUTMAP[`i',6] <0 mat OUTMAP[`i',6]=0
  local  i  =  `i' + 1
  }
  
  display     " "
  display     " Prediction based on the model case(i,t) = popu(i,t)*exp(alpha(i) + beta*t)"
  display     " Prediction for the period=" `tp' ", results below saved in the matrix OUTMAP"
  mat list OUTMAP, f(%6.2f)
//}
	 }  // the end of if "`ppn_'" != "" {

	quietly {	
	//// GRAPHS
	if "`graph'" != "" {
	di " "

	// Age-specific predictions
	replace `peri' = `tperi'  // for graphs peri without possible 0's
	g `obseinci' = `case'/`popu'*10^5
	predict `predinci'
	replace `predinci'=`predinci'/`popu'*10^5
	g `pil'=.
	g `piu'=.
	local iage = 1
	while `iage' <= `nage' {
		local nobs = _N+1 // adding tp for a given age-group
		set obs `nobs'
		replace `peri' = `tp'   in l
		replace `predinci' = OUTMAP[`iage',1]   in l
		replace `age'  = `iage' in l
		replace `pil'  = OUTMAP[`iage',5]/`ppn_'[`iage',1]*10^5 in l // prediction interval down all
		replace `piu'  = OUTMAP[`iage',6]/`ppn_'[`iage',1]*10^5 in l // prediction interval up all
		local iage = `iage' + 1
	}
	sort `age' `peri'
	twoway (line `obseinci' `peri' if `peri'!=`tp') ///
         (line `predinci' `peri' if `peri'!=`tp', clpattern(dash)) ///
			   (scatter `predinci' `peri' if `peri'==`tp', msymbol(+)) ///
			   (rcap `pil' `piu' `peri' if `peri'==`tp'), ///
			  by(`age', title(`title')) xtitle(Period) ytitle(Inci/10^5) saving(predmap_age,replace) ///
				legend( label(1 "observed") label(2 "fitted") order(1 2)) ylabel( ,angle(horizontal))
		if "`agegr'" != "" {
		replace `predinci' = OUTMAP[`agegr',1]   in l
		replace `pil'  = OUTMAP[`agegr',5]/`ppn_'[`agegr',1]*10^5 in l // prediction interval down all
		replace `piu'  = OUTMAP[`agegr',6]/`ppn_'[`agegr',1]*10^5 in l // prediction interval up all
		twoway (line `obseinci' `peri' if `peri'!=`tp' & `age'==`agegr') ///
         (line `predinci' `peri' if `peri'!=`tp' & `age'==`agegr', clpattern(dash)) ///
			   (scatter `predinci' `peri' if `peri'==`tp' & `age'==`agegr', msymbol(+)) ///
			   (rcap `pil' `piu' `peri' if `peri'==`tp' & `age'==`agegr'), ///
					title("`title'  age=`agegr'") xtitle(Period) ytitle(Inci/10^5) ysc(r(0)) saving(predmap_age`agegr',replace) ///
					legend( label(1 "observed") label(2 "fitted") order(1 2)) ylabel( ,angle(horizontal))
		}
		drop if `peri' == `tp'
	//Cases
	replace   `predinci' = `predinci'/10^5
	g   `predcase' = `predinci'*`popu' // predicted cases, musze podzielic na 10^5
	g   `obseadju'   = `pps_'[`age',1]*`case'/`popu'*(10^5) // observed adj-rate
	g   `predadju' = `predinci'*`pps_'[`age',1]*(10^5)       // to calculate pred-hist age-adj
	collapse (sum) `case' `predcase' `predadju' `obseadju', by(`peri')
	local nobs = _N+1 // adding tp
	set obs `nobs'
	replace `predcase' = OUTMAP[`na'+1 , 2] in `nobs'
	replace      `peri' =`tp'  in `nobs' //
	g `pil' = OUTMAP[`na'+1,5] // prediction interval down
	g `piu' = OUTMAP[`na'+1,6] // prediction interval up
	twoway (line `case' `peri' if `peri'!=`tp') (line `predcase' `peri' if `peri'!=`tp', clpattern(dash)) (rcap `pil' `piu' `peri' if `peri'==`tp') ///
	(scatter `predcase' `peri' if `peri'==`tp', msymbol(+)), ysc(r(0)) ylabel( ,angle(horizontal)) ///
	legend( label(1 "observed") label(2 "fitted") ) ytitle(Number of cases) xtitle(Period) saving(predmap_case,replace) /*ylabel(0(`s2')`s1')*/ ///
	title(`title')  legend( label(1 "observed") label(2 "fitted") order(1 2)) // graph for number of cases
	// Adj-rate
	replace   `predadju' = OUTMAP[`nage'+2,  1] in `nobs' // predicted value for adj-rate
	replace      `peri' =`tp'  in `nobs'
	replace `pil' = OUTMAP[`na'+2,5] // prediction interval down
	replace `piu' = OUTMAP[`na'+2,6] // prediction interval up
	local s1=round(OUTMAP[`na'+2,6]*1.3, 1) // numbers gora the graph
	local s2=round(`s1'/4)                // numbers gor the graph
	twoway (line `obseadju'  `peri' if `peri'!=`tp')  (line `predadju' `peri' if `peri'!=`tp', clpattern(dash)) ///
	(rcap `pil' `piu' `peri' if `peri'==`tp') (scatter `predadju' `peri' if `peri'==`tp', msymbol(+)), ///
	ytitle(Adjusted-Rate/10^5) xtitle(Period) ylabel( ,angle(horizontal)) saving(predmap_adju,replace) ///
	ysc(r(0)) title(`title') legend( label(1 "observed") label(2 "fitted") order(1 2)) 
	if "`agegr'" != "" noi di "Graphs saved in the files: predmap_case.gph predmap_adju.gph predmap_age.gph predmap_age`agegr'.gph"
		else noi di "Graphs saved in the files: predmap_case.gph predmap_adju.gph predmap_age.gph.gph"
	} // graphs
	if "`file'" != "" {
		drop _all
		mat coln OUTMAP = inci10_5 cases se_expe se_pred pred95_l pred95_u
		qui svmat OUTMAP, n(col)
		qui g str10 what = ""
		local i = 1
		while `i' <= `nage' {
			qui replace what = "age`i'" in `i'
			local i = `i'+1
	}
	loc  i = _N-1
	qui replace what = "all_ages" in `i'
	loc  i = `i'+1
	qui replace what = "adj10_5" in `i'
	order what
	noi di "Results saved in the file `file'.dta"
	qui save `file', replace
		}
}
   restore
   end
