  /* c(i,t) = popu(i,t)*alpha(i)*( 1 + beta*t )
  Dyba T, Pivrinta L, Hakulinen T,  A simple non-linear model in incidence prediction. Statistics in Medicine 16: 2297--2309, 1997.
  Example of using it prednap case_vari person-year_vari , age(age_vari) peri(peri_vari) ppn(prognostic-population_column-matrix) pps(prognostic-standard_column-matrix) tp(time-of-prediction_number) 
  you can also add optionally graph option and title(Your text for graph) if you want to get the graphs of estimated predictions, the graphs will be saved in the files named: prednap_case.gph and prednap_adju.gph, then the command will look like:
  prednap case_vari person-year_vari , age(age_vari) peri(peri_vari) ppn(prognostic-population_column-matrix) pps(prognostic-standard_column-matrix) tp(time-of-prediction_number) graph title(Your text for graph)
  you can also use if and in statement for the command (prednap case_vari person-year_vari if... in..., ...) but if you do so don't forget to adjust matrices prognostic-population_column-matrix and prognostic-standard_column-matrix for the new number of age groups */
  
  program define prednap /**/
  version 8.0
  syntax varlist(min=2 max=2) [if] [in] , age(string) peri(string) [cons(numlist) ppn(string) pps(string) tp(string) graph title(string) agegr(string) file(string)]
  tokenize "`varlist'"
  args case popu
	tempname h1 s1 s2 ppn_ pps_
	tempvar tperi ycase predcase obseadju predadju pil piu obseinci predinci devi pear_chi2



  preserve
  display     " "
  capture keep  `if' `in'
  // Changing age code from whatever it is to 1,2,3,4,..
  qui inspect `age'
  local nage=r(N_unique) // how many age-groups
  qui tabu `age', gen(_nage)
  local i = 1
  while `i' <= `nage' {
  qui replace `age' = `i' if _nage`i'==1
  local i = `i'+1
  } 

	g `tperi' = `peri'  // to save original code of period for graphs
  //for age groups with constant incidence
   	if "`cons'" != "" {
   		local cl = wordcount("`cons'") // number of age groups with constant incidence
   		forvalues i = 1/`cl'  {	// trick with peri==0 for each age group
   				local ca = word("`cons'",`i')	// index of the age group
					qui replace `peri' = 0 if `age'==`ca'
					qui replace `case' = 0.01 if `case' == 0 & `age'==`ca'
   		}
   	}
	// To define own parametrization
	local i = 1
	while `i' <= `nage' {
	g _age_`i'        = ( `age' == `i' )
	local i = `i' + 1
  } 
	qui ml model lf _prednap (age: `case' `popu' = _age* , noconstant) (period: `peri',noconstant)	 
	qui ml search, repeat(1000)
	ml maximize, difficult iterate(2000)
	
	//Fitted values
	mat PE= e(b)  /*Parameter Estimated*/
	g `predinci' = PE[1,`age']*(1 + PE[1,`nage'+1]*`peri')*`popu'
  //Deviance
  egen `devi'=sum( 2*(`case'*ln(`case'/`predinci')+(`case'-`predinci')) )
	egen `pear_chi2'=sum( (`case'-`predinci')^2/`predinci' )
	display "scaled deviance = " `devi'
	display "Pearson's chi2 = "`pear_chi2'
	display     " "
	di " p-value for Pearsons' chi2= " chi2tail(_N-`nage'-1,`pear_chi2'[1])
//	global pval_Pear = chi2tail(_N-`nage'-1,`pear_chi2'[1]) /*saving after fit */

  //PREDICTION
  if "`ppn'" != "" {
		matrix `ppn_' = `ppn'
		matrix `pps_' = `pps'
		global over_rati = 0
		// possible overdispersion
    if chi2tail(_N-`nage'-1,`pear_chi2'[1]) < 0.05   {
			di " Results adjusted for overdispersion, dispersion factor=" `pear_chi2'[1]/(_N-`nage'-1)
			global over_rati = `pear_chi2'[1]/(_N-`nage'-1)
		}
	//Calculating prediction
  matr `h1' = `pps_'' * J( rowsof(`pps_'), 1, 1)  /*norming standard population*/
  scala `h1'=`h1'[1,1]                          /*norming standard population*/
  matr `pps_' = `pps_'/`h1'                       /*norming standard population*/
  
  //ouput after fitting
  matr   V  = get( VCE )      /* co-variance matrix */

	global over_rati = 0 // to save overdispersion rate if overdispersion
		if chi2tail(e(df),e(deviance_p)) < 0.05 {
		matr V=V*(e(deviance_p)/e(df)) /*correction for possible overdispersion*/
		global over_rati = (e(deviance_p)/e(df)) // to save overdispersion rate if overdispersion
	}	
	matr   PE = get( _b )       /* parameter vector   */
  matr   PE = PE'
  local   na  = rowsof(PE)-1  /* number of age groups */
  
  //Creating matrix of results with names of rows and columns
  mat OUTNAP=J( `na' + 2, 6, .)
  local outname "age1"
  local i  2
  while `i' <=  `na' {
  local outname `outname'  age`i'
  local  i  =  `i' + 1
  }
  
  local outname   `outname' ages-combined  adj-rate/10^5
  matrix rownames  OUTNAP  = `outname'  
  matrix colnames   OUTNAP   =  inci/10^5 no-cases  se(expe-value)  se(pred-value)  Low95%PredInte   Up
  
  //Calculating prediction
		//age-specific results
  local i=1
	while `i' <=  `na' { //*poczatek wyrzuca beta*t dla constant grup wiekowych
		local mn = 1 // *mnoznik 0/1   
 		if "`cons'" != "" { // *by tp nie znieksztalcilo wyniku dla stalych grup wiekowych
		local cl = wordcount("`cons'") // *number of age groups with constant incidence
   	forvalues j = 1/`cl'  {	// *trick with peri==0 for each age group
   		local ca = word("`cons'",`j')	// *index of the age group
			if `i' == `ca' local mn = 0 // gdy stala grupa to mnoznik jest 0
   	}
  }

	mat    OUTNAP[`i', 1] = PE[`i',1]*(1 + PE[`na' + 1,1] * `tp'*`mn')	// incidence, part *`peri'[`i'] to include the option cons()
  mat    OUTNAP[`i', 2] = OUTNAP[`i', 1] * `ppn_'[`i',1]	        	/*cases    */
  scal `s1' = OUTNAP[`i',2]/PE[`i',1]					/*to calculate se of age-specific pred */
  scal `s2' = `ppn_'[`i',1]*PE[`i',1]*`tp'					/*to calculate se of age-specific pred */
  mat   OUTNAP[`i' , 3] =  sqrt(  V[ `i' , `i' ]*(`s1'^2) + V[`i', `na' + 1]*`s1'*`s2'*2 + V[`na' + 1, `na' + 1] * (`s2'^2)  )  /*se of expe age-specific pred*/
  mat   OUTNAP[`i' , 4] =  sqrt(  OUTNAP[`i' , 3]^2  +  OUTNAP[`i', 2]    ) /*se of pred age-specific pred*/
  mat   OUTNAP[`i' , 5] =  OUTNAP[`i' , 2] - 1.96*OUTNAP[`i' , 4]  		  /*95% PredInter of age-specific pred*/
  mat   OUTNAP[`i' , 6] =  OUTNAP[`i' , 2] + 1.96*OUTNAP[`i' , 4]  		  /*95% PredInter of age-specific pred*/
  local  i  =  `i' + 1
  }

	///OVERALL RESULTS
  mat   OUTNAP[`na'+1 , 2]  =  OUTNAP[1..`na', 2]' * J(`na',1,1)/* exp for cases overall  */
  matrix  `h1' = `ppn_'' * J(`na',1,1)				 		/* inci   overall  */
  scalar `h1' = `h1'[1,1]                                        /* inci           overall */
  mat   OUTNAP[`na'+1,  1]  =  OUTNAP[`na'+1 , 2] /`h1'          /* inci           overall */
  mat   OUTNAP[`na'+2,  1]  =  OUTNAP[1..`na' , 1]' * `pps_'      /* exp for adj-rate       */
		//stanard errors by delta method
		//for cases
  mat  `h1' = `ppn_'*(1 + PE[`na' + 1,1] * `tp')\((`ppn_''*PE[1..`na',1])*`tp')	 	/*partial derivatives for cases */
		// part.deriv for cons for cases
	if "`cons'" != "" {
		local cl = wordcount("`cons'") // number of age groups with constant incidence
			forvalues i = 1/`cl'  {	// trick with peri==0 for each age group
				local ca = word("`cons'",`i')	// index of the age group
				mat `h1'[`ca',1] = `ppn_'[`ca',1] // deriv_age
				mat `h1'[`na'+1,1] = `h1'[`na'+1,1] - `ppn_'[`ca',1]*PE[`ca',1]*`tp' // deriv_beta, odejm bo mialo byc 0 w sumie dla cons age 
			}
	}

	mat  OUTNAP[`na'+1, 3] = sqrt(trace(`h1'*(`h1'')*V))           	/* se of exp for cases         */
  mat  OUTNAP[`na'+1, 4] = sqrt(OUTNAP[`na'+1, 3]^2+OUTNAP[`na'+1,2])  /* se of prediction for cases  */
		//for adj-rate
  mat  `h1' = `pps_'*(1+PE[`na'+1,1]*`tp') 	/*partial derivatives for adj-rate*/
	mat  `h1' = `h1'\(`tp'*(`pps_''*PE[1..`na',1]))		/*partial derivatives for adj-rate*/
	// part.deriv for cons for adj
	if "`cons'" != "" {
		local cl = wordcount("`cons'") // number of age groups with constant incidence
			forvalues i = 1/`cl'  {	// trick with peri==0 for each age group
				local ca = word("`cons'",`i')	// index of the age group
				mat `h1'[`ca',1] = `pps_'[`ca',1] // deriv_age
				mat `h1'[`na'+1,1] = `h1'[`na'+1,1] - `pps_'[`ca',1]*PE[`ca',1]*`tp' // deriv_beta, odejm bo mialo byc 0 w sumie dla cons age 
			}
	}
	 
	mat  OUTNAP[`na'+2, 3] = sqrt(trace(`h1'*(`h1'')*V))  			/* se of exp for adj-rate*/
		//loop necessary to calculate se of predi for adjrate <-- STATA cannot divide matrices routinely
  local i=1
  local h1=0
  while `i' <=  `na' {
  local h1=`h1'+ `pps_'[`i',1]^2/`ppn_'[`i',1]*OUTNAP[`i',1]
  local  i  =  `i' + 1
  }
  
  mat OUTNAP[`na'+2, 4] = sqrt(OUTNAP[`na'+2, 3]^2+`h1')  	/* se of prediction for adj-rate*/
		//95% Prediction Intervals
  mat  OUTNAP[`na'+1,5]=OUTNAP[`na'+1,2]-1.96*OUTNAP[`na'+1,4] /*95%PI for cases   */
  mat  OUTNAP[`na'+1,6]=OUTNAP[`na'+1,2]+1.96*OUTNAP[`na'+1,4] /*95%PI for cases   */
  mat  OUTNAP[`na'+2,5]=OUTNAP[`na'+2,1]-1.96*OUTNAP[`na'+2,4] /*95%PI for adj-rate*/
  mat  OUTNAP[`na'+2,6]=OUTNAP[`na'+2,1]+1.96*OUTNAP[`na'+2,4] /*95%PI for adj-rate*/
  
		//All results for age-specific incidences and adj-rates per 10^5 person-years
  local i=1
  while `i' <=  (`na'+1) {
  mat OUTNAP[`i',1]=OUTNAP[`i',1]*10^5
  local  i  =  `i' + 1
  }
  local i=1
  while `i' <=  6 {
  mat OUTNAP[`na'+2,`i']=OUTNAP[`na'+2,`i']*10^5
  local  i  =  `i' + 1
  }
		//Correction for minus
  local i=1
  while `i' <=  (`na'+2) {
  if OUTNAP[`i',5] <0 mat OUTNAP[`i',5]=0
  if OUTNAP[`i',6] <0 mat OUTNAP[`i',6]=0
  local  i  =  `i' + 1
  }
  display     " "
  display     " Prediction based on the model c(i,t) = popu(i,t)*alpha(i)*( 1 + beta*t )"
  display     " Prediction for the period=" `tp' ", results below saved in the matrix OUTNAP"
  mat list OUTNAP, f(%6.2f)
	di ""
  }  // the end of if "`ppn_'" != "" {
  
	quietly {
	////GRAPHS
	if "`graph'" != "" {
	di " "
		//Age-specific predictions
	replace `peri' = `tperi'  // for graphs peri without possible 0's
	g `obseinci' = `case'/`popu'*10^5
	replace `predinci' = `predinci'/`popu'*10^5
	g `pil'=.
	g `piu'=.
	local iage = 1
	while `iage' <= `nage' {
		local nobs = _N+1 // adding tp for a given age-group
		set obs `nobs'
		replace `peri' = `tp'   in l
		replace `predinci' = OUTNAP[`iage',1]   in l
		replace `age'  = `iage' in l
		replace `pil'  = OUTNAP[`iage',5]/`ppn_'[`iage',1]*10^5 in l // prediction interval down all
		replace `piu'  = OUTNAP[`iage',6]/`ppn_'[`iage',1]*10^5 in l // prediction interval up all
		local iage = `iage' + 1
	}
	sort `age' `peri'
	twoway (line `obseinci' `peri' if `peri'!=`tp') ///
         (line `predinci' `peri' if `peri'!=`tp', clpattern(dash)) ///
			   (scatter `predinci' `peri' if `peri'==`tp', msymbol(+)) ///
			   (rcap `pil' `piu' `peri' if `peri'==`tp'), ///
			  by(`age', title(`title')) xtitle(Period) ytitle(Inci/10^5) saving(prednap_age,replace) ///
				legend( label(1 "observed") label(2 "fitted") order(1 2)) ylabel( ,angle(horizontal))
	if "`agegr'" != "" {
	replace `predinci' = OUTNAP[`agegr',1]   in l
	replace `pil'  = OUTNAP[`agegr',5]/`ppn_'[`agegr',1]*10^5 in l // prediction interval down all
	replace `piu'  = OUTNAP[`agegr',6]/`ppn_'[`agegr',1]*10^5 in l // prediction interval up all
	twoway (line `obseinci' `peri' if `peri'!=`tp' & `age'==`agegr') ///
       (line `predinci' `peri' if `peri'!=`tp' & `age'==`agegr', clpattern(dash)) ///
		   (scatter `predinci' `peri' if `peri'==`tp' & `age'==`agegr', msymbol(+)) ///
		   (rcap `pil' `piu' `peri' if `peri'==`tp' & `age'==`agegr'), ///
				title("`title' age=`agegr'") xtitle(Period) ytitle(Inci/10^5) ysc(r(0)) saving(prednap_age`agegr',replace) ///
				legend( label(1 "observed") label(2 "fitted") order(1 2)) ylabel( ,angle(horizontal))
	}
	drop if `peri' == `tp'

		//Cases
	replace   `predinci' = `predinci'/10^5
	g   `predcase' = `predinci'*`popu' // predicted cases, musze podzielic na 10^5
	g   `obseadju'   = `pps_'[`age',1]*`case'/`popu'*(10^5) // observed adj-rate
	g   `predadju' = `predinci'*`pps_'[`age',1]*(10^5)       // to calculate pred-hist age-adj
	collapse (sum) `case' `predcase' `predadju' `obseadju', by(`peri')
	local nobs = _N+1 // adding tp
	set obs `nobs'
	replace `predcase' = OUTNAP[`na'+1 , 2] in `nobs'
	replace      `peri' =`tp'  in `nobs' //
	g `pil' = OUTNAP[`na'+1,5] // prediction interval down
	g `piu' = OUTNAP[`na'+1,6] // prediction interval up
	twoway (line `case' `peri' if `peri'!=`tp') (line `predcase' `peri' if `peri'!=`tp', clpattern(dash)) (rcap `pil' `piu' `peri' if `peri'==`tp') ///
	(scatter `predcase' `peri' if `peri'==`tp', msymbol(+)), ysc(r(0)) ylabel( ,angle(horizontal)) ///
	legend( label(1 "observed") label(2 "fitted") ) ytitle(Number of cases) xtitle(Period) saving(prednap_case,replace) /*ylabel(0(`s2')`s1')*/ ///
	title(`title')  legend( label(1 "observed") label(2 "fitted") order(1 2)) // graph for number of cases
		//Adj-rate
	replace   `predadju' = OUTNAP[`nage'+2,  1] in `nobs' // predicted value for adj-rate
	replace      `peri' =`tp'  in `nobs'
	replace `pil' = OUTNAP[`na'+2,5] // prediction interval down
	replace `piu' = OUTNAP[`na'+2,6] // prediction interval up
	local s1=round(OUTNAP[`na'+2,6]*1.3, 1) // numbers gora the graph
	local s2=round(`s1'/4)                // numbers gor the graph
	twoway (line `obseadju'  `peri' if `peri'!=`tp')  (line `predadju' `peri' if `peri'!=`tp', clpattern(dash)) ///
	(rcap `pil' `piu' `peri' if `peri'==`tp') (scatter `predadju' `peri' if `peri'==`tp', msymbol(+)), ///
	ytitle(Adjusted-Rate/10^5) xtitle(Period) ylabel( ,angle(horizontal)) saving(prednap_adju,replace) ///
	ysc(r(0)) title(`title') legend( label(1 "observed") label(2 "fitted") order(1 2)) 
	if "`agegr'" != "" noi di "Graphs saved in the files: prednap_case.gph prednap_adju.gph prednap_age.gph prednap_age`agegr'.gph"
		else noi di "Graphs saved in the files: prednap_case.gph prednap_adju.gph prednap_age.gph.gph"
	}
	} //graphs

	if "`file'" != "" {
		drop _all
		mat coln OUTNAP = inci10_5 cases se_expe se_pred pred95_l pred95_u
		qui svmat OUTNAP, n(col)
		qui g str10 what = ""
		local i = 1
		while `i' <= `nage' {
			qui replace what = "age`i'" in `i'
			local i = `i'+1
	}
	loc  i = _N-1
	qui replace what = "all_ages" in `i'
	loc  i = `i'+1
	qui replace what = "adj10_5" in `i'
	order what
	di "Results also saved in the file `file'.dta"
	qui save `file', replace
		}
	restore
  end
